/*******************************************************************************
* FILE NAME: user_routines_sensor_test_fast.c <VEX VERSION>
*
* DESCRIPTION:
*  This file contains the code to handle the Sonar Sensor.  It uses a timer to
*  measure the value of the PWM pulse from the sensor.  Since the data is somewhat
*  unstable at various distances, it uses a simple filter to throw out irregular 
*  data.
*
* USAGE:
*  You can either modify this file to fit your needs, or remove it from your 
*  project and replace it with a modified copy. 
*
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "user_routines.h"
#include "printf_lib.h"

extern void Handle_Sensors(void);

/*** DEFINE USER VARIABLES AND INITIALIZE THEM HERE ***/
extern unsigned char myval;
unsigned char HiResCounter;
unsigned char LowResCounter;
unsigned char fltHiResCounter;
unsigned char fltLowResCounter;
unsigned char fltCounter;
unsigned char SonarDataValid;
unsigned char Delta;
bitid EncoderDataFlag;
unsigned char EncoderDataValue;
unsigned char EncoderDataTickCounter;
unsigned char TimeCounter;

void Handle_Timer2_Interrupt(void); /* Used for Shaft Encoder Sensor */
void Handle_Encoder_Interrupt(void);
void Handle_EchoPulse_From_Sonar_Device(void);

#pragma code InterruptVectorLow = LOW_INT_VECTOR
void InterruptVectorLow (void)
{
  _asm
    goto InterruptHandlerLow  /*jump to interrupt routine*/
  _endasm
}

#pragma code
#pragma interruptlow InterruptHandlerLow save=PROD /* You may want to save additional symbols. */

void InterruptHandlerLow ()     
{                               
  unsigned char int_byte; 
  
  rc_dig_out11 ^= 1;

  if (INTCON3bits.INT2IF)         /* The INT2 pin is RB2/INTERRUPTS 1. (rc_dig_int1) */
  { 
    INTCON3bits.INT2IF = 0;
    Handle_EchoPulse_From_Sonar_Device();
  }
  else if (INTCON3bits.INT3IF)    /* The INT3 pin is RB3/INTERRUPTS 2. (rc_dig_int2) */
  {
    INTCON3bits.INT3IF = 0;
    Handle_Encoder_Interrupt();
  }
  else if (INTCONbits.RBIF)       /* INTERRUPTS 3-4 (RB4, RB5, RB6, or RB7) changed. */
  {
    int_byte = PORTB;             /* You must read or write to PORTB   */
    INTCONbits.RBIF = 0;          /* and clear the interrupt flag      */
  }                               /* to clear the interrupt condition. */
  else if ( (INTCONbits.TMR0IE) && (INTCONbits.TMR0IF) )
  {
    INTCONbits.TMR0IF = 0;
    HiResCounter++;
  }
  else if ( (PIE1bits.TMR2IE) && (PIR1bits.TMR2IF) )
  {
    PIR1bits.TMR2IF = 0;
    Handle_Timer2_Interrupt();
  }
}

void Handle_Timer2_Interrupt(void) /* Used for Shaft Encoder Sensor */
{
  TimeCounter++;
  if (TimeCounter > 1)
  {
    EncoderDataFlag.bit0 = 0;     /* indicate Timer 2 timeout occured */         
    rc_dig_out16 = 0;             /* Turn LED4 off */
    rc_dig_out09 ^= 1;
    PIE1bits.TMR2IE = 0;          /* disable TMR2 ints */
  }
}

void Handle_Encoder_Interrupt(void)
{
  EncoderDataTickCounter++;
  if (EncoderDataFlag.bit0)
  {
    EncoderDataValue = TMR2;
    EncoderDataFlag.bit1 = 1;   /* indicate that data is ready to read */
    rc_dig_out16 = 1;           /* Turn LED4 on */
  }
  rc_dig_out07 ^= 1;
  TimeCounter = 0;              /* Clear timer counter */
  TMR2 = 0;                     /* reset timer registers */
  PIR1bits.TMR2IF = 0;          /* make sure int flag is clear */
  EncoderDataFlag.bit0 = 1;     /* indicate that Timer 2 has started */      
  PIE1bits.TMR2IE = 1;          /* enable TMR2 ints */
}

void Handle_EchoPulse_From_Sonar_Device(void)
{
  if (INTCON2bits.INTEDG2)      /* rising edge ? */
  {                             /* Yes */
    T0CONbits.T0PS0 = 1;        /* Change scale to DIV 8 (204us delay)*/
    TMR0H = TMR0L = 0;          /* reset timer registers */
    INTCONbits.TMR0IF = 0;      /* make sure int flag is clear */
    INTCONbits.TMR0IE = 1;      /* Enable TMR0 Int */
    INTCON2bits.INTEDG2 = 0;    /* trigger of falling edge */
    HiResCounter = LowResCounter = 0;
  }
  else
  {
    INTCON3bits.INT2IE = 0;     /* disable rc_dig_int1 for now */
    LowResCounter = TMR0L;
    INTCONbits.TMR0IE = 0;      /* disable TMR0 Int */

    /* Now apply a filter to data */
    if (HiResCounter < 50)
    {
      if (fltHiResCounter == HiResCounter) 
      {
        SonarDataValid = 1;
        fltCounter = 0;
        Delta = fltLowResCounter - LowResCounter;
        if (Delta & 0x80)   /* If negative then negate it */
          Delta = !Delta;
        if (Delta > 20)
          fltCounter++;
      }
      else
      {
        fltCounter++;
      }

      if (fltCounter > 3)
      {
         SonarDataValid = 1;
         fltHiResCounter = HiResCounter;
         fltLowResCounter = LowResCounter;
      }
    }
  }
}

void User_Autonomous_Code(void)
{
  /* Initialize all PWMs and Relays when entering Autonomous mode, or else it
     will be stuck with the last values mapped from the joysticks.  Remember, 
     even when Disabled it is reading inputs from the Operator Interface. 
  */
  pwm01 = pwm02 = pwm03 = pwm04 = 127;
  pwm05 = pwm06 = pwm07 = pwm08 = 127;
  pwm09 = pwm10 = pwm11 = pwm12 = 127;

  while (autonomous_mode)   /* DO NOT CHANGE! */
  {
    if (statusflag.NEW_SPI_DATA)      /* 26.2ms loop area */
    {
      Getdata(&rxdata);   /* DO NOT DELETE, or you will be stuck here forever! */

      Handle_Sensors();

      Putdata(&txdata);   /* DO NOT DELETE, or you will get no PWM outputs! */
    }
  }
}


void Process_Data_From_Local_IO(void)
{
  /* Add code here that you want to be executed every program loop. */

}


/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
